#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2008-2010 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

import sys
import os
from os.path import isfile, isdir
import shutil
import time
from skoollib import SkoolGame, SkoolDaze, BackToSkool
from skoolhtml import join, FileInfo, SkoolWriter, SkoolDazeWriter, BackToSkoolWriter

def notify(notice):
    if verbose:
        print notice

def clock(operation, prefix, *args):
    if verbose:
        sys.stdout.write('%s ' % prefix)
        sys.stdout.flush()
        go = time.clock()
    result = operation(*args)
    if verbose:
        sys.stdout.write('(%0.2fs)\n' % (time.clock() - go))
    return result

def parse_options(options):
    verbose = True
    modes = 'sb'
    skoolfile = None
    files = ''
    all_files = 'bCcdGghikLlmpSsty'
    i = 0
    while i < len(options):
        option = options[i]
        if option == '-q':
            verbose = False
        elif option == '-b':
            modes = 'b'
        elif option == '-s':
            modes = 's'
        elif option == '-f':
            skoolfile = options[i + 1]
            modes = 'u'
            i += 1
        elif option[:2] == '-w':
            files += option[2:]
        i += 1
    return verbose, modes, skoolfile, files or all_files

#
# Begin...
#
if len(sys.argv) < 2:
    sys.stderr.write("Usage: %s [options] <output-dir>\n\n" % sys.argv[0])
    sys.stderr.write("Options:\n")
    sys.stderr.write("  -q       Be quiet\n")
    sys.stderr.write("  -s       Write Skool Daze files only\n")
    sys.stderr.write("  -b       Write Back to Skool files only\n")
    sys.stderr.write("  -f FILE  Use FILE instead of sd.skool or bts.skool\n")
    sys.stderr.write("  -wX      Write only these files, where X is one or more of:\n")
    sys.stderr.write("             b = Bugs\n")
    sys.stderr.write("             C = Changelog\n")
    sys.stderr.write("             c = Character buffers\n")
    sys.stderr.write("             d = Disassembly files\n")
    sys.stderr.write("             G = Game status buffer\n")
    sys.stderr.write("             g = Graphics files\n")
    sys.stderr.write("             h = Home page\n")
    sys.stderr.write("             I = Disassembly index (minimal)\n")
    sys.stderr.write("             i = Disassembly index (full)\n")
    sys.stderr.write("             k = Keypress table\n")
    sys.stderr.write("             L = Lessons and personal timetable index\n")
    sys.stderr.write("             l = Load code\n")
    sys.stderr.write("             m = Memory maps\n")
    sys.stderr.write("             p = Pokes\n")
    sys.stderr.write("             S = Startup code\n")
    sys.stderr.write("             s = Save code\n")
    sys.stderr.write("             t = Trivia\n")
    sys.stderr.write("             y = Glossary\n")
    sys.exit(1)

verbose, modes, uskoolfile, files = parse_options(sys.argv[1:-1])
pdir, bdir = os.path.split(sys.argv[-1])
if not bdir:
    pdir, bdir = os.path.split(pdir)
topdir = join(pdir, bdir)

# Create the top-level directory if necessary
if not isdir(topdir):
    notify('Creating directory %s' % topdir)
    os.makedirs(topdir)

# Copy style.css and skool.js if necessary
for f in (FileInfo.css, FileInfo.js):
    if isfile(f):
        ts = os.stat(f).st_mtime
        html_f = join(topdir, f)
        if not isfile(html_f) or ts > os.stat(html_f).st_mtime:
            notify('Copying %s to %s' % (f, topdir))
            shutil.copy(f, topdir)

if not files.replace('C', '').replace('h', ''):
    # Don't parse a skool file if just writing the changelog or the home page
    modes = ''

for mode in modes:
    if mode == 's':
        # Set things up for Skool Daze
        skoolfile = FileInfo.sd_skool
        reffiles = (FileInfo.skool_ref, FileInfo.sd_ref)
        skool_class = SkoolDaze
        skool_writer_class = SkoolDazeWriter
        game_dir = FileInfo.sd_dir
    elif mode == 'b':
        # Set things up for Back to Skool
        skoolfile = FileInfo.bts_skool
        reffiles = (FileInfo.skool_ref, FileInfo.bts_ref)
        skool_class = BackToSkool
        skool_writer_class = BackToSkoolWriter
        game_dir = FileInfo.bts_dir
    else:
        # Set things up for an unknown skool file
        skoolfile = uskoolfile
        prefix = uskoolfile[:-6]
        reffile = '%s.ref' % prefix
        reffiles = (reffile,) if isfile(reffile) else ()
        skool_class = SkoolGame
        skool_writer_class = SkoolWriter
        game_dir = prefix

    # Parse the skool file and initialise the writer
    skool = clock(skool_class, 'Parsing %s' % skoolfile, skoolfile, reffiles)
    file_info = FileInfo(topdir, game_dir)
    skool_writer = skool_writer_class(skool, file_info)

    # Create the disassembly subdirectory if necessary
    odir = skool_writer.file_info.odir
    if not isdir(odir):
        notify('  Creating directory %s' % odir)
        os.makedirs(odir)

    # Write logo image file if necessary
    if not skool_writer.file_exists(FileInfo.logo):
        clock(skool_writer.write_logo_image, '  Writing %s' % join(game_dir, FileInfo.logo))

    # Write index.html
    if 'i' in files and mode in 'sb':
        clock(skool_writer.write_index, '  Writing %s' % join(game_dir, FileInfo.index))

    # Write disassembly files
    if 'd' in files:
        clock(skool_writer.write_asm_entries, '  Writing disassembly files in %s' % join(game_dir, FileInfo.asm_path))

    # Write the memory map files
    if 'm' in files:
        clock(skool_writer.write_memory_map, '  Writing %s' % join(game_dir, FileInfo.memory_map))
        clock(skool_writer.write_routines_map, '  Writing %s' % join(game_dir, FileInfo.routines))
        clock(skool_writer.write_data_map, '  Writing %s' % join(game_dir, FileInfo.data))
        clock(skool_writer.write_messages_map, '  Writing %s' % join(game_dir, FileInfo.messages))

    # Write graphics files
    if 'g' in files and mode in 'sb':
        clock(skool_writer.write_playarea, '  Writing %s' % join(game_dir, FileInfo.playarea))
        clock(skool_writer.write_playarea_tiles, '  Writing %s' % join(game_dir, FileInfo.playarea_tiles))
        clock(skool_writer.write_asstart, '  Writing %s' % join(game_dir, FileInfo.asstart))
        clock(skool_writer.write_animatory_states, '  Writing %s' % join(game_dir, FileInfo.animatory_states))
        clock(skool_writer.write_animatory_state_tiles, '  Writing %s' % join(game_dir, FileInfo.animatory_state_tiles))
        clock(skool_writer.write_graphics, '  Writing %s' % join(game_dir, FileInfo.graphics))
        clock(skool_writer.write_graphic_glitches, '  Writing %s' % join(game_dir, FileInfo.graphic_glitches))

    # Write lesson files and personal timetables index
    if 'L' in files and mode in 'sb':
        clock(skool_writer.write_lessons, '  Writing lesson files in %s' % join(game_dir, FileInfo.lessons_path))
        clock(skool_writer.write_lesson_index, '  Writing %s' % join(game_dir, FileInfo.lesson_index))
        clock(skool_writer.write_personal_timetable_index, '  Writing %s' % join(game_dir, FileInfo.personal_timetables))

    # Write other files
    if 'G' in files and skool_writer.has_gbuffer():
        clock(skool_writer.write_gbuffer, '  Writing %s' % join(game_dir, FileInfo.gbuffer))
    if 'b' in files and skool_writer.has_bugs():
        clock(skool_writer.write_bugs, '  Writing %s' % join(game_dir, FileInfo.bugs))
    if 't' in files and skool_writer.has_facts():
        clock(skool_writer.write_facts, '  Writing %s' % join(game_dir, FileInfo.facts))
    if 'y' in files and skool_writer.has_glossary():
        clock(skool_writer.write_glossary, '  Writing %s' % join(game_dir, FileInfo.glossary))
    if 'p' in files and skool_writer.has_pokes():
        clock(skool_writer.write_pokes, '  Writing %s' % join(game_dir, FileInfo.pokes))
    if 'c' in files and mode in 'sb':
        clock(skool_writer.write_character_buffers, '  Writing %s' % join(game_dir, FileInfo.cbuffer))
    if 'k' in files and mode in 'sb':
        clock(skool_writer.write_keytable, '  Writing %s' % join(game_dir, FileInfo.keys))
    if 'S' in files and mode in 'sb':
        skoolfile = FileInfo.start_skool
        skool2 = clock(skool_class, '  Parsing %s' % skoolfile, skoolfile, reffiles)
        skool2_writer = skool_writer_class(skool2, file_info)
        clock(skool2_writer.write_start_routine, '    Writing %s' % join(game_dir, FileInfo.asm_relpath('', skool2.start_address)))
    if 'l' in files and mode in 'sb':
        skoolfile = FileInfo.load_skool
        skool2 = clock(skool_class, '  Parsing %s' % skoolfile, skoolfile, reffiles)
        skool2_writer = skool_writer_class(skool2, file_info)
        clock(skool2_writer.write_load_entries, '    Writing disassembly files in %s' % join(game_dir, FileInfo.load_path))
        clock(skool2_writer.write_load_map, '    Writing %s' % join(game_dir, FileInfo.load_index))
    if 's' in files and mode in 'sb':
        skoolfile = FileInfo.save_skool
        skool2 = clock(skool_class, '  Parsing %s' % skoolfile, skoolfile, reffiles)
        skool2_writer = skool_writer_class(skool2, file_info)
        clock(skool2_writer.write_save_routine, '    Writing %s' % join(game_dir, FileInfo.save))

    # Write minimal index.html
    if 'I' in files or ('i' in files and mode not in 'sb'):
        clock(skool_writer.write_index, '  Writing minimal %s' % join(game_dir, FileInfo.index), True)

if uskoolfile is None:
    if 'h' in files:
        clock(SkoolWriter.write_home_page, 'Writing %s' % join(topdir, FileInfo.home), topdir)
    if 'C' in files:
        clock(SkoolWriter.write_changelog, 'Writing %s' % join(topdir, FileInfo.changelog), topdir)
